@@ -60,26 +60,10 @@
 namespace papilo
 {
 
-template <typename REAL, bool isfptype = num_traits<REAL>::is_floating_point>
-struct RealParseType
-{
-   using type = double;
-};
-
-template <typename REAL>
-struct RealParseType<REAL, true>
-{
-   using type = REAL;
-};
-
 /// Parser for mps files in fixed and free format
 template <typename REAL>
 class MpsParser
 {
-   static_assert(
-       num_traits<typename RealParseType<REAL>::type>::is_floating_point,
-       "the parse type must be a floating point type" );
-
  public:
    static boost::optional<Problem<REAL>>
    loadProblem( const std::string& filename )
@@ -121,10 +105,6 @@ class MpsParser
          problem.set_problem_type( ProblemFlag::kInteger );
       }
 
-      problem.setInputTolerance(
-          REAL{ pow( typename RealParseType<REAL>::type{ 10 },
-                     -std::numeric_limits<
-                         typename RealParseType<REAL>::type>::digits10 ) } );
       return problem;
    }
 
@@ -212,6 +192,12 @@ class MpsParser
 
    ParseKey
    parseBounds( boost::iostreams::filtering_istream& file );
+
+   std::pair<bool, REAL>
+   read_number( const std::string& s );
+
+   REAL
+   pow( int base, int exponent );
 };
 
 template <typename REAL>
@@ -390,13 +376,19 @@ MpsParser<REAL>::parseCols( boost::iostreams::filtering_istream& file,
          assert( -1 == rowidx );
    };
 
-   auto addtuple = [&rowidx, &ncols,
-                    this]( typename RealParseType<REAL>::type coeff ) {
+   auto addtuple = [&rowidx, &ncols, this]( std::string sval) {
+      auto result = read_number(sval);
+      if(result.first)
+      {
+         fmt::print("could not parse {}\n", sval);
+         exit(0);
+      }
+      REAL coeff = result.second;
       if( rowidx >= 0 )
          entries.push_back(
-             std::make_tuple( ncols - 1, rowidx, REAL{ coeff } ) );
+             std::make_tuple( ncols - 1, rowidx, coeff ) );
       else
-         coeffobj.push_back( std::make_pair( ncols - 1, REAL{ coeff } ) );
+         coeffobj.push_back( std::make_pair( ncols - 1,  coeff ) );
    };
 
    while( getline( file, strline ) )
@@ -488,13 +480,20 @@ MpsParser<REAL>::parseCols( boost::iostreams::filtering_istream& file,
 
       assert( ncols > 0 );
 
-      if( !qi::phrase_parse(
-              it, strline.end(),
-              +( qi::lexeme[qi::as_string[+qi::graph][( parsename )]] >>
-                 qi::real_parser<typename RealParseType<REAL>::type>()[(
-                     addtuple )] ),
-              ascii::space ) )
+      std::istringstream is( strline );
+      std::vector<std::string> tokens;
+      std::string tmp;
+      while( is >> tmp )
+         tokens.push_back( tmp );
+      if( tokens.size() != 3 && tokens.size() != 5 )
          return ParseKey::kFail;
+      parsename( tokens[1] );
+      addtuple( tokens[2] );
+      if( tokens.size() == 5 )
+      {
+         parsename( tokens[3] );
+         addtuple( tokens[4] );
+      }
    }
 
    return ParseKey::kFail;
@@ -532,8 +531,14 @@ MpsParser<REAL>::parseRanges( boost::iostreams::filtering_istream& file )
          assert( rowidx >= 0 && rowidx < nRows );
       };
 
-      auto addrange = [&rowidx,
-                       this]( typename RealParseType<REAL>::type val ) {
+      auto addrange = [&rowidx, this]( std::string sval ) {
+         auto result = read_number(sval);
+         if(result.first)
+         {
+            fmt::print("could not parse {}\n", sval);
+            exit(0);
+         }
+         REAL val = result.second;
          assert( size_t( rowidx ) < rowrhs.size() );
 
          if( row_type[rowidx] == BoundType::kGE )
@@ -565,22 +570,20 @@ MpsParser<REAL>::parseRanges( boost::iostreams::filtering_istream& file )
          }
       };
 
-      // compulsory part
-      if( !qi::phrase_parse(
-              it, strline.end(),
-              +( qi::lexeme[qi::as_string[+qi::graph][( parsename )]] >>
-                 qi::real_parser<typename RealParseType<REAL>::type>()[(
-                     addrange )] ),
-              ascii::space ) )
+      std::istringstream is( strline );
+      std::vector<std::string> tokens;
+      std::string tmp;
+      while( is >> tmp )
+         tokens.push_back( tmp );
+      if( tokens.size() != 3 && tokens.size() != 5 )
          return ParseKey::kFail;
-
-      // optional part todo don't replicate code
-      qi::phrase_parse(
-          it, strline.end(),
-          +( qi::lexeme[qi::as_string[+qi::graph][( parsename )]] >>
-             qi::real_parser<typename RealParseType<REAL>::type>()[(
-                 addrange )] ),
-          ascii::space );
+      parsename( tokens[1] );
+      addrange( tokens[2] );
+      if( tokens.size() == 5 )
+      {
+         parsename( tokens[3] );
+         addrange( tokens[4] );
+      }
    }
 
    return ParseKey::kFail;
@@ -618,7 +621,14 @@ MpsParser<REAL>::parseRhs( boost::iostreams::filtering_istream& file )
          assert( rowidx < nRows );
       };
 
-      auto addrhs = [&rowidx, this]( typename RealParseType<REAL>::type val ) {
+      auto addrhs = [&rowidx, this]( std::string sval ) {
+         auto result = read_number(sval);
+         if(result.first)
+         {
+            fmt::print("could not parse {}\n", sval);
+            exit(0);
+         }
+         REAL val = result.second;
          if( rowidx == -1 )
          {
             objoffset = -REAL{ val };
@@ -641,19 +651,20 @@ MpsParser<REAL>::parseRhs( boost::iostreams::filtering_istream& file )
          }
       };
 
-      // Documentation Link to qi:
-      // https://www.boost.org/doc/libs/1_66_0/libs/spirit/doc/html/spirit/qi/tutorials/warming_up.html
-      // +: Parse a one or more times
-      // lexeme[a]: Disable skip parsing for a, does pre-skipping
-      // as_string: Force atomic assignment for string attributes
-      // graph: Matches a character based on the equivalent of std::isgraph in the current character set
-      if( !qi::phrase_parse(
-              it, strline.end(),
-              +( qi::lexeme[qi::as_string[+qi::graph][( parsename )]] >>
-                 qi::real_parser<typename RealParseType<REAL>::type>()[(
-                     addrhs )] ),
-              ascii::space ) )
+      std::istringstream is( strline );
+      std::vector<std::string> tokens;
+      std::string tmp;
+      while( is >> tmp )
+         tokens.push_back( tmp );
+      if( tokens.size() != 3 && tokens.size() != 5 )
          return ParseKey::kFail;
+      parsename( tokens[1] );
+      addrhs( tokens[2] );
+      if( tokens.size() == 5 )
+      {
+         parsename( tokens[3] );
+         addrhs( tokens[4] );
+      }
    }
 
    return ParseKey::kFail;
@@ -739,8 +750,7 @@ MpsParser<REAL>::parseBounds( boost::iostreams::filtering_istream& file )
       }
 
       // parse over next word
-      qi::phrase_parse( it, strline.end(), qi::lexeme[+qi::graph],
-                        ascii::space );
+      qi::phrase_parse( it, strline.end(), qi::lexeme[+qi::graph], ascii::space );
 
       int colidx;
 
@@ -780,40 +790,52 @@ MpsParser<REAL>::parseBounds( boost::iostreams::filtering_istream& file )
          continue;
       }
 
-      if( !qi::phrase_parse(
-              it, strline.end(),
-              +( qi::lexeme[qi::as_string[+qi::graph][( parsename )]] >>
-                 qi::real_parser<typename RealParseType<REAL>::type>()[(
-                     [&ub_is_default, &lb_is_default, &colidx, &islb, &isub,
-                      &isintegral,
-                      this]( typename RealParseType<REAL>::type val ) {
-                        if( islb )
-                        {
-                           lb4cols[colidx] = REAL{ val };
-                           lb_is_default[colidx] = false;
-                           col_flags[colidx].unset( ColFlag::kLbInf );
-                        }
-                        if( isub )
-                        {
-                           ub4cols[colidx] = REAL{ val };
-                           ub_is_default[colidx] = false;
-                           col_flags[colidx].unset( ColFlag::kUbInf );
-                        }
+      auto adddomains = [&ub_is_default, &lb_is_default, &colidx, &islb, &isub, &isintegral, this]
+          ( std::string sval )
+      {
+         auto result = read_number(sval);
+         if(result.first)
+         {
+            fmt::print("could not parse {}\n", sval);
+            exit(0);
+         }
+         REAL val = result.second;
+         if( islb )
+         {
+            lb4cols[colidx] = REAL{ val };
+            lb_is_default[colidx] = false;
+            col_flags[colidx].unset( ColFlag::kLbInf );
+         }
+         if( isub )
+         {
+            ub4cols[colidx] = REAL{ val };
+            ub_is_default[colidx] = false;
+            col_flags[colidx].unset( ColFlag::kUbInf );
+         }
 
-                        if( isintegral )
-                           col_flags[colidx].set( ColFlag::kIntegral );
+         if( isintegral )
+            col_flags[colidx].set( ColFlag::kIntegral );
 
-                        if( col_flags[colidx].test( ColFlag::kIntegral ) )
-                        {
-                           col_flags[colidx].set( ColFlag::kIntegral );
-                           if( !islb && lb_is_default[colidx] )
-                              lb4cols[colidx] = REAL{ 0.0 };
-                           if( !isub && ub_is_default[colidx] )
-                              col_flags[colidx].set( ColFlag::kUbInf );
-                        }
-                     } )] ),
-              ascii::space ) )
+         if( col_flags[colidx].test( ColFlag::kIntegral ) )
+         {
+            col_flags[colidx].set( ColFlag::kIntegral );
+            if( !islb && lb_is_default[colidx] )
+               lb4cols[colidx] = REAL{ 0.0 };
+            if( !isub && ub_is_default[colidx] )
+               col_flags[colidx].set( ColFlag::kUbInf );
+         }
+      };
+
+      std::istringstream is( strline );
+      std::vector<std::string> tokens;
+      std::string tmp;
+      while( is >> tmp )
+         tokens.push_back( tmp );
+      if( tokens.size() != 4 )
          return ParseKey::kFail;
+      parsename( tokens[2] );
+      adddomains( tokens[3] );
+
    }
 
    return ParseKey::kFail;
@@ -844,6 +866,78 @@ MpsParser<REAL>::parseFile( const std::string& filename )
    return parse( in );
 }
 
+template <typename REAL>
+REAL
+MpsParser<REAL>::pow(int base, int exponent)
+{
+   REAL answer = 1;
+   for(int i = 0; i < exponent; i++)
+      answer *= 10;
+   return answer;
+}
+
+template <typename REAL>
+std::pair<bool, REAL>
+MpsParser<REAL>::read_number(const std::string& s) {
+   bool failed = false;
+   REAL answer = 0;
+   bool negated = false;
+   bool dot = false;
+   bool exponent = false;
+   int exp = 0;
+   bool exp_negated = false;
+   int digits_after_dot = 0;
+   for (char c : s) {
+      if ('0' <= c && c <= '9') {
+         int digit = c - '0';
+         if(exponent)
+         {
+            exp *= 10;
+            exp += digit;
+         }
+         else if( !dot )
+         {
+            answer *= 10;
+            answer += digit;
+         }
+         else
+         {
+            digits_after_dot++;
+            answer += digit /  pow( 10, digits_after_dot );
+         }
+      }
+      else if (c == '.' && !dot)
+      {
+         assert(digits_after_dot == 0);
+         assert(!exponent);
+         dot = true;
+      }
+      else if (c == '-' && ((exponent && !exp_negated) || !negated))
+      {
+         if(exponent)
+            exp_negated = true;
+         else
+            negated = true;
+      }
+      else if (c == '+' && ((exponent && !exp_negated) || !negated))
+      {
+      }
+      else if( ( c == 'E' || c == 'e' ) && !exponent )
+         exponent = true;
+      else
+      {
+         failed = true;
+         assert(false);
+      }
+   }
+   if( !exp_negated )
+      answer *= pow( 10,  exp );
+   else
+      answer /= pow( 10,  exp );
+   return {failed, negated ? -answer : answer};
+}
+
+
 template <typename REAL>
 bool
 MpsParser<REAL>::parse( boost::iostreams::filtering_istream& file )
